# Copyright 2015 Jared Rodriguez (jared.rodriguez@rackspace.com)
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mercury_agent.inspector.inspectors import inspector
from mercury_agent.inspector.hwlib import lspci


@inspector.expose('pci')
def pci_inspector():
    _pci = lspci.parse_nnvmmk()
    return _pci


if __name__ == '__main__':
    from pprint import pprint

    pprint(pci_inspector())

