# Copyright 2015 Jared Rodriguez (jared.rodriguez@rackspace.com)
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
from setuptools import setup, find_packages


setup(
    name='mercury-agent',
    version='0.1.8',
    packages=find_packages(exclude=['tests']),
    url='http://www.mercurysoft.io',
    license='Apache-2.0',
    author='Jared Rodriguez',
    author_email='jared.rodriguez@rackspace.com',
    description='Mercury agent, hardware libraries, and inspectors',
    install_requires=[
        'mercury-core',
        'netifaces',
        'pyudev',
        'python-hpssa',
        'lxml',
        'press',
        'six',
        'msgpack-python',
        'pyzmq'
    ],
    entry_points="""
    [console_scripts]
    mercury-agent = mercury_agent.agent:main
    """,
)
